export declare function parseMode(mode: string, args: any): string;
export declare function modeValidation(args: object): void;
export declare function displayModeHelp(mode: string, args: any): string;
