/**
 * The content of this file is generated by a tool.
 * Don't edit it manually!
 * */
export declare type FileSignatureDetails = {
    versions: string[];
    filename: string;
};
export declare type VulnerableSignatures = {
    [signature: string]: FileSignatureDetails;
};
export declare const vulnerableSignatures: VulnerableSignatures;
