import { MonitorResult } from '../types';
export declare function formatErrorMonitorOutput(packageManager: any, res: MonitorResult, options: any, projectName?: string): string;
export declare function formatMonitorOutput(packageManager: any, res: MonitorResult, options: any, projectName?: string, foundProjectCount?: number): string;
