"use strict";
exports.id = 575;
exports.ids = [575];
exports.modules = {

/***/ 77575:
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
const fs = __webpack_require__(35747);
const path = __webpack_require__(85622);
function about() {
    console.log(`Snyk CLI Open Source Attributions\n\n`);
    const licenseNoticesArray = JSON.parse(fs.readFileSync(path.resolve(__dirname, 'thirdPartyNotice.json'), 'utf8'));
    for (const licenseNotice of licenseNoticesArray) {
        console.log(`${licenseNotice.name} \u00B7 ${licenseNotice.version} \u00B7 ${licenseNotice.license}`);
        console.log(`Author(s): ${licenseNotice.author ||
            'Not filled'} \u00B7 Package: ${licenseNotice.source || ''}`);
        console.log(`${licenseNotice.licenseText || ''}`); // WTFPL is not required the embed its license text
        console.log('\n+-+-+-+-+-+-+');
        console.log('\n');
    }
}
exports.default = about;


/***/ })

};
;
//# sourceMappingURL=575.index.js.map