import { Options, TestOptions } from '../../../lib/types';
export declare function formatDockerBinariesIssues(dockerBinariesSortedGroupedVulns: any, binariesVulns: any, options: Options & TestOptions): string[];
