import { TestConfig } from '../types';
interface LocalCache {
    policyEnginePath: string;
    rulesBundlePath: string;
}
export declare function initLocalCache(testConfig: TestConfig): Promise<LocalCache>;
export {};
