import * as types from './types';
import { SupportedPackageManagers } from '../package-managers';
export declare function loadPlugin(packageManager: SupportedPackageManagers | undefined): types.Plugin;
