import { TestConfig } from '../types';
import { TestOutput } from './results';
export declare const systemCachePath: string;
export declare function scan(options: TestConfig, policyEnginePath: string, rulesBundlePath: string): Promise<TestOutput>;
