import { IaCTestFailure } from '../types';
export declare function formatIacTestFailures(testFailures: IaCTestFailure[]): string;
export declare function formatFailuresList(testFailures: IaCTestFailure[]): string;
