export const languages = [
    {
        "name": "Alpine Abuild",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 14,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "group": "Shell",
        "aliases": [
            "abuild",
            "apkbuild"
        ],
        "filenames": [
            "APKBUILD"
        ],
        "tmScope": "source.shell",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "CODEOWNERS",
        "aceMode": "gitignore",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 321684729,
        "vscodeLanguageIds": [
            "gitignore"
        ],
        "filenames": [
            "CODEOWNERS"
        ],
        "tmScope": "text.codeowners"
    },
    {
        "name": "Dockerfile",
        "aceMode": "dockerfile",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 89,
        "vscodeLanguageIds": [
            "dockerfile"
        ],
        "aliases": [
            "Containerfile"
        ],
        "tmScope": "source.dockerfile",
        "extensions": [
            ".dockerfile"
        ],
        "filenames": [
            "Containerfile",
            "Dockerfile"
        ],
        "codemirrorMode": "dockerfile",
        "codemirrorMimeType": "text/x-dockerfile"
    },
    {
        "name": "Gentoo Ebuild",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 127,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "group": "Shell",
        "extensions": [
            ".ebuild"
        ],
        "tmScope": "source.shell",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "Gentoo Eclass",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 128,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "group": "Shell",
        "extensions": [
            ".eclass"
        ],
        "tmScope": "source.shell",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "Git Attributes",
        "aceMode": "gitignore",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 956324166,
        "vscodeLanguageIds": [
            "gitignore"
        ],
        "group": "INI",
        "aliases": [
            "gitattributes"
        ],
        "filenames": [
            ".gitattributes"
        ],
        "tmScope": "source.gitattributes",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "Ignore List",
        "aceMode": "gitignore",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 74444240,
        "vscodeLanguageIds": [
            "gitignore"
        ],
        "group": "INI",
        "aliases": [
            "ignore",
            "gitignore",
            "git-ignore"
        ],
        "extensions": [
            ".gitignore"
        ],
        "filenames": [
            ".atomignore",
            ".babelignore",
            ".bzrignore",
            ".coffeelintignore",
            ".cvsignore",
            ".dockerignore",
            ".eleventyignore",
            ".eslintignore",
            ".gitignore",
            ".markdownlintignore",
            ".nodemonignore",
            ".npmignore",
            ".prettierignore",
            ".stylelintignore",
            ".vercelignore",
            ".vscodeignore",
            "gitignore-global",
            "gitignore_global"
        ],
        "tmScope": "source.gitignore",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "OpenRC runscript",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 265,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "group": "Shell",
        "aliases": [
            "openrc"
        ],
        "interpreters": [
            "openrc-run"
        ],
        "tmScope": "source.shell",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "Shell",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 346,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "aliases": [
            "sh",
            "shell-script",
            "bash",
            "zsh"
        ],
        "extensions": [
            ".sh",
            ".bash",
            ".bats",
            ".cgi",
            ".command",
            ".env",
            ".fcgi",
            ".ksh",
            ".sh.in",
            ".tmux",
            ".tool",
            ".zsh",
            ".zsh-theme"
        ],
        "filenames": [
            ".bash_aliases",
            ".bash_history",
            ".bash_logout",
            ".bash_profile",
            ".bashrc",
            ".cshrc",
            ".env",
            ".env.example",
            ".flaskenv",
            ".kshrc",
            ".login",
            ".profile",
            ".zlogin",
            ".zlogout",
            ".zprofile",
            ".zshenv",
            ".zshrc",
            "9fs",
            "PKGBUILD",
            "bash_aliases",
            "bash_logout",
            "bash_profile",
            "bashrc",
            "cshrc",
            "gradlew",
            "kshrc",
            "login",
            "man",
            "profile",
            "zlogin",
            "zlogout",
            "zprofile",
            "zshenv",
            "zshrc"
        ],
        "interpreters": [
            "ash",
            "bash",
            "dash",
            "ksh",
            "mksh",
            "pdksh",
            "rc",
            "sh",
            "zsh"
        ],
        "tmScope": "source.shell",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "ShellSession",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 347,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "extensions": [
            ".sh-session"
        ],
        "aliases": [
            "bash session",
            "console"
        ],
        "tmScope": "text.shell-session",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "Tcsh",
        "aceMode": "sh",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "linguistLanguageId": 368,
        "vscodeLanguageIds": [
            "shellscript"
        ],
        "group": "Shell",
        "extensions": [
            ".tcsh",
            ".csh"
        ],
        "interpreters": [
            "tcsh",
            "csh"
        ],
        "tmScope": "source.shell",
        "codemirrorMode": "shell",
        "codemirrorMimeType": "text/x-sh"
    },
    {
        "name": "JvmOptions",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "extensions": [
            ".vmoptions"
        ],
        "filenames": [
            "jvm.options"
        ],
        "vscodeLanguageIds": [
            "jvmoptions"
        ]
    },
    {
        "name": "hosts",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "filenames": [
            "hosts"
        ],
        "vscodeLanguageIds": [
            "hosts"
        ]
    },
    {
        "name": "dotenv",
        "since": "0.1.0",
        "parsers": [
            "sh"
        ],
        "extensions": [
            ".env"
        ],
        "filenames": [
            ".env.*"
        ],
        "vscodeLanguageIds": [
            "dotenv"
        ]
    }
];
//# sourceMappingURL=languages.js.map