export var LangVariant;
(function (LangVariant) {
    LangVariant[LangVariant["LangBash"] = 0] = "LangBash";
    LangVariant[LangVariant["LangPOSIX"] = 1] = "LangPOSIX";
    LangVariant[LangVariant["LangMirBSDKorn"] = 2] = "LangMirBSDKorn";
    LangVariant[LangVariant["LangBats"] = 3] = "LangBats";
})(LangVariant || (LangVariant = {}));
//# sourceMappingURL=types.js.map