export declare function containsSpotlightVulnIds(results: any[]): string[];
declare type VulnerabilityId = string;
export declare function notificationForSpotlightVulns(foundSpotlightVulnsIds: VulnerabilityId[]): string;
export {};
