import { Options, TestOptions, MonitorOptions, PolicyOptions } from './types';
export declare function isMultiProjectScan(options: Partial<Options & TestOptions & MonitorOptions & PolicyOptions>): boolean;
