import * as types from '../types';
import { MultiProjectResult } from '@snyk/cli-interface/legacy/plugin';
export declare function inspect(root: string, targetFile: string, options?: types.Options): Promise<MultiProjectResult>;
