/**
 * The Policy Engine release version associated with this Snyk CLI version.
 */
export declare const policyEngineReleaseVersion: string;
/**
 * The Policy Engine executable's file name.
 */
export declare const policyEngineFileName: string;
export declare const policyEngineChecksum: string;
