import { ScanResult } from '../../../lib/ecosystems/types';
import { TestResult } from '../../../lib/snyk-test/legacy';
export declare function convertLegacyTestResultToScanResult(testResult: TestResult): ScanResult;
