import { Options } from '../../../types';
import { Log } from '../types';
export declare function getCodeDisplayedOutput(codeTest: Log, meta: string, prefix: string): string;
export declare function getMeta(options: Options, path: string): string;
export declare function getPrefix(path: string): string;
