import { IaCTestFlags } from '../../../../cli/commands/test/iac/local-execution/types';
/**
 * Displayed as the title of the test output.
 */
export declare const iacTestTitle: string;
/**
 * Progress indication message while files are tested.
 */
export declare const spinnerMessage: string;
/**
 * Displayed when a test resolves successfully.
 */
export declare const spinnerSuccessMessage: string;
/**
 * Message for using custom rules.
 */
export declare const customRulesMessage: string;
/**
 * Message for using custom rules.
 */
export declare const customRulesReportMessage: string;
/**
 * @returns whether or not to include user messages in the output.
 */
export declare function shouldLogUserMessages(options: IaCTestFlags): boolean;
