import { CustomError } from '../../../../../errors';
import { TestConfig } from '../../types';
export declare function downloadPolicyEngine(testConfig: TestConfig): Promise<string>;
export declare const policyEngineUrl: string;
export declare class FailedToDownloadPolicyEngineError extends CustomError {
    constructor();
}
export declare class FailedToCachePolicyEngineError extends CustomError {
    constructor(savePath: string);
}
