"use strict";
exports.id = 522;
exports.ids = [522];
exports.modules = {

/***/ 1522:
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
const theme = __webpack_require__(86988);
function protectFunc() {
    console.log(theme.color.status.warn(`\n${theme.icon.WARNING} WARNING: Snyk protect was removed at 31 March 2022.\nPlease use '@snyk/protect' package instead: https://updates.snyk.io/snyk-wizard-and-snyk-protect-removal-224137 \n`));
}
exports.default = protectFunc;


/***/ })

};
;
//# sourceMappingURL=522.index.js.map