export declare function sleep(ms: number): Promise<void>;
export declare const reTryMessage = "Tip: Re-run in debug mode to see more information: DEBUG=*snyk* <COMMAND>";
export declare const contactSupportMessage = "If the issue persists contact support@snyk.io";
export declare function testPlatformSupport(): void;
