import { MethodResult } from './types';
export default function ignore(options: any): Promise<MethodResult>;
export declare function ignoreIssue(options: any): Promise<MethodResult>;
export declare function excludeFilePathPattern(options: any): Promise<MethodResult>;
