import { CustomError } from '../../../../../errors';
import { TestConfig } from '../../types';
export declare class InvalidUserPolicyEnginePathError extends CustomError {
    constructor(path: string, message?: string, userMessage?: string);
}
export declare function lookupLocalPolicyEngine(testConfig: TestConfig): Promise<string | undefined>;
