export declare const policyEngineVersion: string;
export declare function formatPolicyEngineFileName(releaseVersion: string): string;
export declare function getChecksum(policyEngineFileName: string): string;
