export declare function api(): string | undefined;
export declare function getOAuthToken(): string | undefined;
export declare function getDockerToken(): string | undefined;
export declare function apiTokenExists(): string;
export declare function apiOrOAuthTokenExists(): string;
export declare function getAuthHeader(): string;
export declare function someTokenExists(): boolean;
