"use strict";
exports.id = 959;
exports.ids = [959];
exports.modules = {

/***/ 55959:
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
const theme = __webpack_require__(86988);
function wizard() {
    console.log(theme.color.status.warn(`\n${theme.icon.WARNING} WARNING: Snyk wizard was removed at 31 March 2022.\nPlease use 'snyk ignore' instead: https://updates.snyk.io/snyk-wizard-and-snyk-protect-removal-224137 \n`));
}
exports.default = wizard;


/***/ })

};
;
//# sourceMappingURL=959.index.js.map