export default function npm(method: string, packages: string[] | null, live: boolean, cwd: string | null, flags: string[] | null): Promise<void>;
export declare function getVersion(): Promise<unknown>;
