export declare function getRepositoryRootForPath(p: string): string;
export declare function getRepositoryRoot(cwd?: string): string;
export declare function getWorkingDirectoryForPath(p: string): string;
