import { MethodArgs } from '../../args';
export declare function findHelpFile(helpArgs: string[], helpFolderPath?: string): string;
export default function help(...args: MethodArgs): Promise<string>;
