export declare function formatShareResultsOutput(orgName: string, projectName: string): string;
export declare function formatShareResultsOutputV2(orgName: string, projectName: string): string;
export declare const shareResultsTip: string;
export declare const shareCustomRulesDisclaimer: string;
