import { IacShareResultsFormat, IaCTestFlags } from '../../cli/commands/test/iac/local-execution/types';
import { ScanResult } from '../ecosystems/types';
import { Policy } from '../policy/find-and-load-policy';
import { IacOutputMeta } from '../types';
export declare function convertIacResultToScanResult(iacResult: IacShareResultsFormat, policy: Policy | undefined, meta: IacOutputMeta, options: IaCTestFlags): ScanResult;
