export declare const PATH_SEPARATOR = " > ";
export declare const PATH_HIDDEN_ELEMENTS = "...";
export declare const CALL_PATH_LEADING_ELEMENTS = 2;
export declare const CALL_PATH_TRAILING_ELEMENTS = 2;
