import { FormattedOutputResultsBySeverity } from '../types';
import { Options } from './types';
export declare function getIacDisplayedIssues(resultsBySeverity: FormattedOutputResultsBySeverity, options?: Options): string;
